import scipy.signal.signaltools as sigtool
import numpy as np

def powerCalculate(Va,Vb,Vc,Ia,Ib,Ic,period):
    va = Va
    vb = Vb
    vc = Vc
    ia = Ia
    ib = Ib
    ic = Ic

    # Measure power for one period (alpha beta domain)
    # suppose decimation 1 : samples every 1us
    numberOfSamplesOnePeriod = period

    valpha = np.zeros(numberOfSamplesOnePeriod)
    vbeta = np.zeros(numberOfSamplesOnePeriod)
    ialpha = np.zeros(numberOfSamplesOnePeriod)
    ibeta = np.zeros(numberOfSamplesOnePeriod)
    Palphabeta = np.zeros(numberOfSamplesOnePeriod)
    Qalphabeta = np.zeros(numberOfSamplesOnePeriod)

    Psum = 0
    Qsum = 0

    for k in range(0,int(numberOfSamplesOnePeriod)):
        valpha[k] = np.sqrt(2/3.0)*(va[k] - (1/2.0)*vb[k] - (1/2.0)*vc[k])
        vbeta[k] = np.sqrt(2/3.0)*((np.sqrt(3)/2.0)*vb[k] - (np.sqrt(3)/2.0)*vc[k])

        ialpha[k] = np.sqrt(2/3.0)*(ia[k] - (1/2.0)*ib[k] - (1/2.0)*ic[k])
        ibeta[k] = np.sqrt(2/3.0)*((np.sqrt(3)/2.0)*ib[k] - (np.sqrt(3)/2.0)*ic[k])

        Palphabeta[k] = valpha[k]*ialpha[k] + vbeta[k]*ibeta[k]
        Qalphabeta[k] = vbeta[k]*ialpha[k] - valpha[k]*ibeta[k]

        Psum += Palphabeta[k]
        Qsum += Qalphabeta[k]

    Pactual = Psum/numberOfSamplesOnePeriod
    Qactual = Qsum/numberOfSamplesOnePeriod

    return (Pactual,Qactual)

def phaseDiff(signala, signalb, onePeriodNumberOfSamples):
    # Calculates phase difference between 2 signals
    # Inputs: Two instantaneous data arrays of same size and one period number of samples.
    #         Both signals must have the same frequency.
    # Returns: Phase difference list of first signal regarding to second.
    
    numberOfSamples = len(signala)
    theta_array = []

    status = True
    if numberOfSamples < onePeriodNumberOfSamples:
        print "ERROR: Number of samples is less than one period number of samples"
        print "%i < %i" % (numberOfSamples, onePeriodNumberOfSamples)
        return False
    else:
        end_calc = int(round(numberOfSamples / onePeriodNumberOfSamples))
        for k in range(0, end_calc + 1):
            start = int(onePeriodNumberOfSamples * k)
            stop = int((onePeriodNumberOfSamples * k + onePeriodNumberOfSamples))

            if stop >= numberOfSamples:
                stop = numberOfSamples
                for i in range(start, stop):
                    theta_array.append(theta)
            else:
                sig1 = signala[start:stop]
                sig2 = signalb[start:stop]
                sig1_float = []
                for i in range(0, len(sig1)):
                    sig1_float.append(float(sig1[i]))

                hilb_sig1 = sigtool.hilbert(sig1_float)
                X_corr = sum(hilb_sig1 * sig2 * np.hanning(len(sig1)))
                theta = -(np.angle(X_corr))

                for i in range(start, stop):
                    theta_array.append(theta)

    return status, theta_array
